--------------------------------------
-- Namespaces
--------------------------------------
local _, core = ...
core.AltVaultProfessionDetails = {}

local AltVault = core.AltVault
local AltVaultProfessionDetails = core.AltVaultProfessionDetails

local MAX_NUM_REAGENTS = 7

local CurrentProfession = nil
local CurrentTier = nil
local CurrentCategory = nil

local qualityToIconBorderAtlas =
{
	[Enum.ItemQuality.Poor] = "auctionhouse-itemicon-border-gray",
	[Enum.ItemQuality.Common] = "auctionhouse-itemicon-border-white",
	[Enum.ItemQuality.Uncommon] = "auctionhouse-itemicon-border-green",
	[Enum.ItemQuality.Rare] = "auctionhouse-itemicon-border-blue",
	[Enum.ItemQuality.Epic] = "auctionhouse-itemicon-border-purple",
	[Enum.ItemQuality.Legendary] = "auctionhouse-itemicon-border-orange",
	[Enum.ItemQuality.Artifact] = "auctionhouse-itemicon-border-artifact",
	[Enum.ItemQuality.Heirloom] = "auctionhouse-itemicon-border-account",
	[Enum.ItemQuality.WoWToken] = "auctionhouse-itemicon-border-account",
};

DetailsFrameMixin = {}

function DetailsFrameMixin:OnShow()
    if CurrentProfession == nil then return end
    PlaySound(SOUNDKIT.IG_CHARACTER_INFO_OPEN);
    local profession = AltVaultDB.characters[CharacterJournal.id].professions[CurrentProfession]

    if profession.name then
        local atlas = "Professions-Recipe-Background-"..profession.name
        self.Title:SetText(profession.name)
        self.BackgroundOverlay:SetAtlas(atlas)
    end
    
end

function DetailsFrameMixin:OnHide()
    ResetState() 
end

function ProfessionDetailsClose(self)
    PlaySound(SOUNDKIT.IG_CHARACTER_INFO_CLOSE);
    self:GetParent():Hide();
end

RecipeFrameMixin = { };

function RecipeFrameMixin:OnLoad()
    local Pad = 0;
    local Spacing = 0;
    local view = CreateScrollBoxListLinearView(Pad, Pad, Pad, Pad, Spacing);
        
    local function Initializer(frame, elementData)
        frame:Init(elementData);
    end

    view:SetElementInitializer("AltVaultProfessionRecipeTemplate", Initializer);
    ScrollUtil.InitScrollBoxWithScrollBar(self.ScrollBox, self.ScrollBar, view);
end

function RecipeFrameMixin:OnShow()
    if CurrentProfession == nil then return end
    -- need to add tutorial box to let people know they can be clicked now
    local recipes = AltVaultDB.characters[CharacterJournal.id].professions[CurrentProfession].recipes
    if recipes == nil or (recipes.current == nil and recipes.legacy == nil) then return end

    if #recipes.current == 0 and #recipes.legacy == 0 then
        self:GetParent().NoRecipes:Show()
        self.ScrollBox:Hide()
        self.ScrollBar:Hide()
    else
        self:GetParent().NoRecipes:Hide()
        -- Add recipes
        local dataProvider = CreateDataProvider();

        if #recipes.current ~= 0 then
            dataProvider:Insert({
                divider = true,
                label = "Dragonflight Recipes"
            });
            for _, recipe in pairs(recipes.current) do
                dataProvider:Insert({
                    id = recipe.recipeID,
                    name = recipe.name,
                    reagents = recipe.reagents,
                    icon = recipe.icon,
                    hyperlink = recipe.hyperlink,
                });
            end
        end
        
        if #recipes.legacy ~= 0 then
            dataProvider:Insert({
                divider = true,
                label = "Legacy Recipes"
            });
            for _, recipe in pairs(recipes.legacy) do
                dataProvider:Insert({
                    id = recipe.recipeID,
                    name = recipe.name,
                    reagents = recipe.reagents,
                    icon = recipe.icon,
                    hyperlink = recipe.hyperlink,
                });
            end
        end

        self.ScrollBox:SetDataProvider(dataProvider, true);
        self.ScrollBox:Show()
        self.ScrollBar:Show()
    end
end

function RecipeFrameMixin:OnHide()
    ResetState()
end

function RecipeFrameMixin:SetCurrentProfession(profession)
    CurrentProfession = profession
end

function RecipeFrameMixin:GetCurrentProfession()
    return CurrentProfession
end

function ResetState()
    CurrentProfession = nil
    CurrentTier = nil
    CurrentCategory = nil
end

-- Recipe
RecipeMixin = { };

function RecipeMixin:Init(elementData)

    if elementData.divider then
        self.Label:SetText(elementData.label)
        self.Label:Show()
        self.LabelBackplate:Show()
        self.Title:Hide()
        self.RecipeIcon:Hide()
        self.Ring:Hide()
        self.NoReagents:Hide()
        for i = 1, MAX_NUM_REAGENTS do
            self["Reagent"..i]:Hide()
        end
    else
        self.Label:Hide()
        self.LabelBackplate:Hide()
        self.Title:SetText(elementData.name)
        self.Title:Show()
        self.RecipeIcon.Icon:SetTexture(elementData.icon)
        self.RecipeIcon:Show()
        self.Ring:Show()

        if elementData.hyperlink then
            local recipeItem = Item:CreateFromItemLink(elementData.hyperlink)
            recipeItem:ContinueOnItemLoad(function()
                local _, _, itemRarity = GetItemInfo(elementData.hyperlink)
                local r, g, b, hex = GetItemQualityColor(itemRarity)
                self.Ring:SetAtlas(qualityToIconBorderAtlas[itemRarity])
                self.Title:SetTextColor(r, g, b, 1)
            end)
    
            self.RecipeIcon:SetScript("OnEnter", function(icon)
                GameTooltip:SetOwner(icon, "ANCHOR_RIGHT")
                GameTooltip:SetHyperlink(elementData.hyperlink);
            end);

            self.RecipeIcon:SetScript("OnClick", function(button, buttonName)
                if ( IsModifiedClick() ) then
                    HandleModifiedItemClick(elementData.hyperlink)	
                end
            end);
        else
            self.Title:SetTextColor(1, 1, 1, 1)
            self.Ring:SetAtlas(qualityToIconBorderAtlas[1])
            self.RecipeIcon:SetScript("OnEnter", function(icon)
                return
            end);
        end
        self.RecipeIcon:SetScript("OnLeave", GameTooltip_Hide);
    
        if elementData.reagents and #elementData.reagents ~= 0 then
            self.NoReagents:Hide()
            for i = 1, MAX_NUM_REAGENTS do
                local reagent = elementData.reagents[i]
                if reagent ~= nil then
                    local reagentItem = Item:CreateFromItemID(reagent.itemID)
                    local currentReagent = self["Reagent"..i]
                    reagentItem:ContinueOnItemLoad(function()
                        local reagentName, reagentHyperlink, reagentRarity, _, _, _, _, _, _, reagentTexture = GetItemInfo(reagent.itemID);
                        currentReagent:SetItemButtonTexture(reagentTexture)
                        SetItemButtonQuality(currentReagent, reagentRarity, nil)
                        currentReagent:SetItemButtonCount(reagent.qtyRequired)
    
                        currentReagent:SetScript("OnEnter", function(button)
                            GameTooltip:SetOwner(button, "ANCHOR_RIGHT")
                            GameTooltip:SetHyperlink(reagentHyperlink);
                        end);
    
                        currentReagent:SetScript("OnLeave", GameTooltip_Hide);
    
                        self["Reagent"..i]:Show()
                    end)
                else
                    self["Reagent"..i]:Hide()
                end
            end
        else
            for i = 1, MAX_NUM_REAGENTS do
                self["Reagent"..i]:Hide()
            end
            self.NoReagents:Show()
        end
    end

	self:Show();
end

